
  function sendNewReport() {
    $('#report_condition').val(JSON.stringify(rule.pack().items));
  }

  function SearchEventIncident(selectData) {
    var self = this;

    self.html = Handlebars.templates['search-rule']({});
    self.columns = selectData.incident_columns || [];
    self.operators = selectData.operators || {};
    self.domains = selectData.domains || { type: "dropdown", value: [] };
    self.statuses = selectData.statuses || { type: "dropdown", value: [] };
    self.sources = selectData.sources || { type: "dropdown", value: [] };
    self.priorities = selectData.priorities || { type: "dropdown", value: [] };
    self.users = selectData.users || { type: "dropdown", value: [] };
    self.init();
    return self;
  }

  SearchEventIncident.prototype = {
    init: function() {
      var self = this;

      self.columns_html = Handlebars.templates['select']({
        name: "column-select",
        placeholder: I18n.t('javascript.choose_query'),
        width: "100%",
        data: { value: self.columns }
      });

      self.domains_html = Handlebars.templates['select']({
        name: "domains-select",
        width: "100%",
        multiple: false,
        data: self.domains
      });

      self.priorities_html = Handlebars.templates['select']({
        name: "priorities-select",
        width: "100%",
        multiple: false,
        data: self.priorities
      });

      self.statuses_html = Handlebars.templates['select']({
        name: "statuses-select",
        width: "100%",
        multiple: false,
        data: self.statuses
      });

      self.sources_html = Handlebars.templates['select']({
        name: "sources-select",
        width: "100%",
        multiple: false,
        data: self.sources
      });

      self.users_html = Handlebars.templates['select']({
        name: "users-select",
        width: "100%",
        multiple: false,
        data: self.users
      });

      self.operators_html = function($html, data) {
        var select = Handlebars.templates['select']({
          name: "operators-select",
          width: "100%",
          data: { value: data }
          });
        $html.find('div.operator-select').html(select);
      };

    },

    searchUI: function(data, callback) {
      var self = this;
      $('#content #title').after(Handlebars.templates['search'](data));

      $('.search-content-remove').on('click', function(e) {
        e.preventDefault();
        self.remove(this);
      });

      $('div.search-content-enable input').on('click', function() {
        var box = $(this).parents('.search-content-box');
        var enabled = $(this).is(':checked');
        box.find('.value *, .operator-select *, .column-select *')
          .attr('disabled', !enabled)
          .css('opacity', enabled ? 1 : 0.8);
        $('.add_chosen').trigger("chosen:updated");
      });

      $('button.submit-search').on('click', function(e) {
        e.preventDefault();
        self.submit();
      });

      $('#content #title').on('click', 'a.reset-search-form', function(e) {
        e.preventDefault();
        $('.rules').empty();
        self.add(); self.add();
      });

      if (callback && typeof callback === "function") callback();
    },

    add: function(that, newOptions, callback) {
      var self = this;
      var $html = $(self.html);
      $html.find('div.column-select').html(self.columns_html);
      self.operators_html($html, self.operators.text_input);

      if (that) {
        $(that).parents('.search-content-box').after($html);
      } else {
        $('.rules').append($html);
      }

      $html.find('.add_chosen').chosen({ allow_single_deselect: false, width: "100%" })
        .change(function() {
          var value = $(this).val();
          var box = $(this).parents('.search-content-box');
          if (value === "domain") {
            box.find('.value').html(self.domains_html);
            self.operators_html($html, self.operators.select);
          } else if (value === "status"){
            box.find('.value').html(self.statuses_html);
            self.operators_html($html, self.operators.select);
          } else if (value === "source"){
            box.find('.value').html(self.sources_html);
            self.operators_html($html, self.operators.select);
          } else if (value === "user"){
            box.find('.value').html(self.users_html);
            self.operators_html($html, self.operators.select);
          }else if (value === "priority"){
            box.find('.value').html(self.priorities_html);
            self.operators_html($html, self.operators.select);
          }
          else {
            box.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" type="text">');
            self.operators_html($html, self.operators.text_input);
          }

          box.find('.add_chosen').chosen({ allow_single_deselect: false, width: "100%" });
          if (callback && typeof callback === "function") callback(box);
          box.trigger('rbsearch:search:change');
        });

      return $html;
    },

    remove: function(that) {
      var count = $('.search-content-box').length;
      if (count > 2 || (count === 2 && $(that).parents('.search-content-box').index() !== 0)) {
        $(that).parents('.search-content-box').remove();
      }
    },

    build: function(search) {
      var self = this;
      var data = typeof search === "string" ? JSON.parse(search) : search || {};    
      
      var rules = data || {};

      for (var id in rules) {
        var item = rules[id];
        var rule = self.add();
        
        var column = rule.find('div.column-select select');

        rule.attr('data-rule-id', id);

        column.find("option[value='" + item.column + "']").prop('selected', true).trigger("chosen:updated").trigger('change');

        var operator = rule.find('div.operator-select select');
        operator.find("option[value='" + item.operator + "']")
          .prop('selected', true)
          .trigger("chosen:updated");
        

        var value = rule.find('div.value select, div.value input');
        value.find("option[value='" + item.value + "']")
          .prop('selected', true)
          .trigger("chosen:updated");
        
        rule.bind('rbsearch:search:change', function() {
          var op = $(this).find('div.operator-select select');
          var val = $(this).find('div.value select, div.value input');
          op.find("option[value='" + item.operator + "']").prop('selected', true).trigger("chosen:updated");
          val.val(item.value);
          rule.unbind('rbsearch:search:change');
        });
        

        if (item.enabled === "false") {
          rule.find('div.search-content-enable input').prop('checked', false);
          rule.find('.value *, .operator-select *, .column-select *')
            .attr('disabled', true).css('opacity', 0.8);
        }
      }

      $('.rules .add_chosen').trigger("chosen:updated");
    },

    submit: function(callback) {
      var self = this;
      var search = self.pack();
      var csrf = $('meta[name="csrf-token"]').attr('content');

      if (callback) callback(search, self);

      if (!$.isEmptyObject(search.items)) {
        post_to_url('/filters/ips/advanced', {
          match_all: search.match_all,
          search: search.items,
          authenticity_token: csrf
        });
      } else {
        flash_message.push({ type: 'error', message: I18n.t('javascript.you_cannot') });
        flash();
      }
    },

    pack: function() {
      var self = this;
      var matchAll = $('select.global-match-setting').val() === "all";
      var json = { match_all: matchAll, items: {} };

      $('.search-content-box').each(function(index, item) {
        var $item = $(item);
        if ($item.find('.search-content-enable input').is(':checked')) {
          var column = $item.find('.column-select select').val();
          var operator = $item.find('.operator-select select').val();
          var value = $item.find('.value input, .value select').val();

          if (column && value) {
            json.items[index] = {
            column: column,
            operator: operator,
            value: value,
            enabled: true
            };
          }
        }
      });

      return json;
    }
  };

  // Listener to changes on the days selector; -->
  $(document).on('click', ".incident-days-selector li", function(event){
    var element = $(event.target);
    var day = parseInt(element.attr("data-day"));

    if(element.hasClass("active")){
      element.removeClass("active");
    } else {
      element.addClass("active");
    }

    updateIncidentDaysInput();

  });

  function updateIncidentDaysInput(){
      var input = $("#incident_group_report_schedule_days");

      var days = [];

      $(".incident-days-selector:not(.hidden) li.active").each(function(index, element){

          days.push($(element).attr("data-day"));

      });

      input.val(days.join(","));

  }

  $(document).on('change', '#incident_group_report_schedule_type', function(event){
      var select = $(event.target);
      var optionSelected = $('option:selected', select).val();

      if(optionSelected == "weekly"){

          $(".incident-days-selector.weekly").removeClass("hidden");
          $(".incident-days-selector.monthly").addClass("hidden");
          $("#day-selector-info").removeClass("hidden");

      } else if (optionSelected == "monthly") {

          $(".incident-days-selector.weekly").addClass("hidden");
          $(".incident-days-selector.monthly").removeClass("hidden");
          $("#day-selector-info").removeClass("hidden");

      } else {
        $(".incident-days-selector.weekly").addClass("hidden");
        $(".incident-days-selector.monthly").addClass("hidden"); 
        $("#day-selector-info").addClass("hidden");
      }

      updateDaysInput();
  });