DittoCode::HoldFile.if 'ENTERPRISE'
#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Provide API methods to work with organizations
#
class Api::V1::InventoryDevicesController < Api::V1::BaseSensorsController
  # ----------------------------------------------
  # Before and after actions
  # ----------------------------------------------
  # skip_before_action :authenticate_user_from_token!, :user_setup_api

  #
  # Return the list of inventory devices.
  #
  def index
    return error_422("You don\'t have privileges to do that.") unless current_user.super_admin?

    @sensor_names = current_user.domains_with_permission.map { |d| [d.id, d.path_name] }.to_h
    @inventory_devices = InventoryDevice.where(sensor_id: @sensor_names.keys)

    render json: {
      success: true,
      inventory_devices: @inventory_devices.map { |inventory_device| { id: inventory_device.id, name: inventory_device.name, sources: inventory_device.sources, first_seen: inventory_device.first_seen, last_seen: inventory_device.last_seen, created_at: inventory_device.created_at, updated_at: inventory_device.updated_at, comments: inventory_device.comments, locked: inventory_device.locked, sensor_id: inventory_device.sensor_id, sensor_name: inventory_device.sensor&.name, ips: inventory_device.ips, operating_system_object_id: inventory_device.operating_system_object_id, operating_system_short_name: inventory_device.operating_system_object&.short_name,  hardware_object_id: inventory_device.hardware_object_id, hardware_short_name: inventory_device.hardware_object&.short_name, criticality_id: inventory_device.criticality_id, criticality_name: inventory_device.criticality&.name, inventory_device_type_object_id: inventory_device.inventory_device_type_object_id, inventory_device_type_object_name: inventory_device.inventory_device_type_object&.name } }
    }
  end

  # 
  # Return the information of an inventory device.
  #
  def show
    return error_422("You don\'t have privileges to do that.") unless current_user.super_admin?

    @sensor_names = current_user.domains_with_permission.map { |d| [d.id, d.path_name] }.to_h
    @inventory_devices = InventoryDevice.where(sensor_id: @sensor_names.keys)
    @inventory_device = @inventory_devices.find_by(id: params[:id])

    if @inventory_device.nil?
      render json: { success: false, message: "Inventory Device not found." }, status: :not_found
    else
      render json: {
        success: true,
        inventory_device: {
          id: @inventory_device.id,
          name: @inventory_device.name,
          sources: @inventory_device.sources,
          first_seen: @inventory_device.first_seen,
          last_seen: @inventory_device.last_seen,
          created_at: @inventory_device.created_at,
          updated_at: @inventory_device.updated_at,
          comments: @inventory_device.comments,
          locked: @inventory_device.locked,
          sensor_id: @inventory_device.sensor_id,
          sensor_name: @inventory_device.sensor&.name,
          ips: @inventory_device.ips,
          operating_system_object_id: @inventory_device.operating_system_object_id,
          operating_system_short_name: @inventory_device.operating_system_object&.short_name,
          hardware_object_id: @inventory_device.hardware_object_id,
          hardware_short_name: @inventory_device.hardware_object&.short_name,
          criticality_id: @inventory_device.criticality_id,
          criticality_name: @inventory_device.criticality&.name,
          inventory_device_type_object_id: @inventory_device.inventory_device_type_object_id,
          inventory_device_type_object_name: @inventory_device.inventory_device_type_object&.name
        }
      }
    end
  end

  #
  # Return incidents of an inventory device
  #
  def incidents
    return error_422("You don\'t have privileges to do that.") unless current_user.super_admin?

    @sensor_names = current_user.domains_with_permission.map { |d| [d.id, d.path_name] }.to_h
    @inventory_devices = InventoryDevice.where(sensor_id: @sensor_names.keys)
    @inventory_device = @inventory_devices.find_by(id: params[:id])

    if @inventory_device.nil?
      render json: { success: false, message: "Inventory Device not found." }, status: :not_found
    else
      render json: {
        success: true,
        inventory_device_incidents: @inventory_device.incidents.map { |incident| { id: incident.id, name: incident.name, visible: incident.visible, source: incident.source, incident_status: incident.incident_status&.name, priority: incident.priority&.name, created_at: incident.created_at, updated_at: incident.updated_at } }
      }
    end
  end

  #
  # Search for inventory devices.
  #
  def search
    return error_422("You don't have privileges to do that.") unless current_user.super_admin?
  
    # Si no hay ningún parámetro de búsqueda, devolver vacío
    if params[:name].blank?
      return render json: { success: true, inventory_devices: [] }
    end
  
    @sensor_names = current_user.domains_with_permission.map { |d| [d.id, d.path_name] }.to_h
    @inventory_devices = InventoryDevice.where(sensor_id: @sensor_names.keys)
  
    if params[:name].present?
      @inventory_devices = @inventory_devices.where("name LIKE ?", "%#{params[:name]}%")
    end
  
    render json: {
      success: true,
      incidents: @inventory_devices.map do |inventory_device|
        {
          id: inventory_device.id,
          name: inventory_device.name,
          sources: inventory_device.sources,
          first_seen: inventory_device.first_seen,
          last_seen: inventory_device.last_seen,
          created_at: inventory_device.created_at,
          updated_at: inventory_device.updated_at,
          comments: inventory_device.comments,
          locked: inventory_device.locked,
          sensor_id: inventory_device.sensor_id,
          sensor_name: inventory_device.sensor&.name,
          ips: inventory_device.ips,
          operating_system_object_id: inventory_device.operating_system_object_id,
          operating_system_short_name: inventory_device.operating_system_object&.short_name,
          hardware_object_id: inventory_device.hardware_object_id,
          hardware_short_name: inventory_device.hardware_object&.short_name,
          criticality_id: inventory_device.criticality_id,
          criticality_name: inventory_device.criticality&.name,
          inventory_device_type_object_id: inventory_device.inventory_device_type_object_id,
          inventory_device_type_object_name: inventory_device.inventory_device_type_object&.name
        }
      end
    }
  end

end
