#######################################################################
# Copyright (c) 2025 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module InventoryDevicesHelper

  def inventory_devices_domains
    inventory_device_sensor_ids = InventoryDevice.distinct.pluck(:sensor_id)

    current_user.domains_with_permission
                              .where(id: inventory_device_sensor_ids)
                              .map { |d| [d.id, d.path_name] }
                              .to_h
  end

  def inventory_devices_type_objects_order_by_name
    object_types = InventoryDeviceTypeObject.all.order(:name)

    object_types.map { |d| [d.id, d.name] }.to_h
  end

  def inventory_devices_order_by
    [[I18n.t("inventory_devices.order_by.id"), 'id'],
     [I18n.t("inventory_devices.order_by.name"), 'name'],
     [I18n.t("inventory_devices.order_by.sources"), 'sources'],
     [I18n.t("inventory_devices.order_by.created_at"), 'created_at'],
     [I18n.t("inventory_devices.order_by.sensor_id"), 'sensor_id'],
     [I18n.t("inventory_devices.order_by.last_seen"), 'last_seen']]
  end

  def inventory_devices_order_direction
    [[I18n.t("inventory_devices.order_direction.asc"), 'asc'],
     [I18n.t("inventory_devices.order_direction.desc"), 'desc']]
  end

  def inventory_devices_uniq_sources
    InventoryDevice.pluck(:sources).flatten.uniq
  end

  def criticality_color(priority_name)
    case priority_name.downcase
    when "low"
      "#89a54e"
    when "medium"
      "#c69958"
    when "high"
      "#aa4643"
    when "critical"
      "#ff4643"
    when "none"
      "#30bf93"
    when "unknow"
      "#919191"
    when "info"
      "#5eca5e"
    when "warning"
      "#ffa500"
    when "notice"
      "#3399ff"
    when "error"
      "#ff0000"
    when 'emergency'
      "#b71c1c"
    when 'alert'
      "#f44336"
    when 'debug'
      "#b2ebf2"
    when 'unset'
      "#000000"
    else
      "#000000"
    end
  end

  # def inventory_device_image(inventory_device_type_object)
  #   type = inventory_device_type_object.try(:name)
  #   default_path = 'icons/assets/default_asset.svg'
  #   file_path = (type && type.is_a?(String) && type.present?) ? "icons/assets/#{type.strip.downcase.gsub(/\s+/, '_')}.svg" : default_path

  #   view_context ||= self
  #   asset_exists?(file_path) ? view_context.asset_path(file_path) : view_context.asset_path(default_path)
  # end

  def inventory_device_image(inventory_device_type_object)
    type = inventory_device_type_object.try(:name)
    default_path = 'icons/assets/default_asset.svg'
    file_path = (type && type.is_a?(String) && type.present?) ? "icons/assets/#{type.strip.downcase.gsub(/\s+/, '_')}.svg" : default_path
    
    svg = asset_exists?(file_path) ? File.read(Rails.root.join('app/assets', 'images', file_path)) : File.read(Rails.root.join('app/assets', 'images', default_path))
    
    # Retorna el SVG con una clase para el estilo dinámico
    svg.html_safe
  end

  def inventory_devices_dropdown_options
    html = ""
    if current_user.super_admin?
      html =
      content_tag(:li) do
        link_to t("import_csv"),
                import_inventory_devices_path,
                class: "import_inventory_device",
                "data-toggle" => "modal",
                "data-target" => "#default_modal",
                "data-backdrop" => "static"
      end
    end

    html +=
      content_tag(:li) do
        link_to t("export_csv"),
                export_inventory_devices_path,
                class: "export_inventory_devices"
      end

    html.html_safe
  end

  def update_inventory_devices_job_executing?
    Delayed::Job.where("handler LIKE ? AND failed_at IS NULL AND locked_at IS NOT NULL", '%Redborder::Jobs::UpdateInventoryDevicesJob%').exists?
  end
end