# app/services/incident_report_generator.rb
class IncidentReportGenerator

  include IncidentSetters

  def initialize()
    # create an instance of ActionView, so we can use the render method outside of a controller
    @av = ActionView::Base.new
    @av.view_paths = ActionController::Base.view_paths
    # need these in case your view constructs any links or references any helper methods.
    @av.class_eval do
      include Rails.application.routes.url_helpers
      include ApplicationHelper
      include DashboardHelper
      include WidgetsHelper
      include ReportBlocksHelper
      include ManagersHelper
      include IncidentsHelper
    end
  end

  def generate_report_html(incident, incident_number)
    @incident = incident
    @incident_number = incident_number
    set_elements
    load_actions_and_worklog_data
    calculate_nodes

    @time = Time.now
    @time_week_ago = Time.now - 7.days
    #@time_month_ago = Time.now - 1.month
    @indicators = load_indicators(@elements)

    assigned_users_ids = IncidentUser.where(incident_id: @incident.id).pluck(:user_id)
    @assigned_users_info = User.where(id: assigned_users_ids).pluck(:name, :email).to_h

    start_date = @time_week_ago.strftime("%d-%m-%y")
    end_date = @time.strftime("%d-%m-%y")
    
    @report_name = get_report_name start_date, end_date, @incident
    @incident_report = @incident.incident_reports

    Rails.application.routes.default_url_options = {host: Setting.find_by(name: 'webpage_url').try(:value)}

    html_content = @av.render(template: 'incidents/incident_report.pdf.erb', 
    locals: { 
    incident: @incident,
    incident_report: @incident_report,
    time: @time,
    report_name: @report_name,
    assigned_users_info: @assigned_users_info,
    indicators: @indicators,
    observables_by_type: @observables_by_type,
    observables: @observables,
    actions_taken_data: @actions_taken_data,
    assets: @assets,
    incident_number: @incident_number,
    page: 0
    })

    return html_content

  end

  def generate_front_page_pdf(report_name, time, assigned_users_info)
    front_page = @av.render(template: 'incidents/incident_report_front_page.pdf.erb',
      locals: {
        report_name: report_name,
        time: time,
        assigned_users_info: assigned_users_info
      })

    grover_front = Grover.new(
      front_page,
      orientation: "Portrait",
      margin: { top: 24, left: 48, right: '1in', bottom: '1in' },
      format: "A4",
      scale: 0.8,
      print_backgroud: true
    )
    grover_front
  end

  def get_report_name start_date, end_date, incident = nil, report_name = nil
    added_name = ''
    if report_name.nil?
      aux_org_name = Sensor.where(id:incident.sensor_id).pluck(:property)
      first_org_name = aux_org_name[0]['path'].split('/').first

      added_name = first_org_name.to_s.strip.empty? ? '' : first_org_name
    else
      added_name = report_name
    end

    return 'RB' + "#{end_date}" + "#{@org_name}" if start_date.nil?
    return 'RB' + "#{start_date}" + '/' + "#{end_date}" + " #{added_name}"
  end

  def generate_footer
    @av.render(partial: 'incidents/incident_report_footer', formats: [:html])
  end

  def generate_main htmls
    @av.render(template: 'incidents/incident_group_report_main.pdf.erb', formats: [:html], locals: {incidents: htmls})
  end

  def generate_group_pdf report, user, report_name = nil
    @incidents = report.filter_by_conditions user

    incident_number = 1

    assigned_users_ids = IncidentUser.where(incident_id: @incidents).pluck(:user_id)
    assigned_users_info = User.where(id: assigned_users_ids).pluck(:name, :email).to_h

    time = Time.now

    time_past = case report.schedule_type
      when "daily"
        Time.now - 1.days
      when "weekly"
        Time.now - 7.days
      when "monthly"
        Time.now - 1.months
      end
    start_date = time_past.strftime("%d-%m-%y")
    end_date = time.strftime("%d-%m-%y")
    
    htmls = []

    @incidents.each do | incident |

      report_data = incident.incident_reports.download_report(incident_number)
      htmls.push report_data
      incident_number += 1
    end

    grover_front = generate_front_page_pdf report_name, time, assigned_users_info
    reports_html = generate_main htmls
    footer_html = generate_footer

    grover = Grover.new(
      reports_html,
      javascript: true,
      display_header_footer: true,
      header_template: '<div></div>',
      footer_template: footer_html,
      orientation: "Portrait",
      margin: { top: 24, left: 48, right: '1in', bottom: '1in' },
      format: "A4",
      scale: 0.8,
      print_backgroud: true
    )

    pdf = CombinePDF.new
    pdf << CombinePDF.parse(grover_front.to_pdf)
    pdf << CombinePDF.parse(grover.to_pdf)
    pdf = pdf.to_pdf
    pdf
  end

  def generate_individual_pdf incident
    @time = Time.now

    assigned_users_ids = IncidentUser.where(incident_id: incident.id).pluck(:user_id)
    @assigned_users_info = User.where(id: assigned_users_ids).pluck(:name, :email).to_h

    end_date = @time.strftime("%d-%m-%y")

    report_data = [incident.incident_reports.download_report(1)]

    report_name = get_report_name nil, end_date, incident
    grover_front = generate_front_page_pdf report_name, @time, @assigned_users_info
    report_html = generate_main(report_data)
    footer_html = generate_footer

    grover = Grover.new(
      report_html,
      display_header_footer: true,
      header_template: '<div></div>',
      footer_template: footer_html,
      orientation: "Portrait",
      margin: { top: 24, left: 48, right: '1in', bottom: '1in' },
      format: "A4",
      scale: 0.8,
      print_backgroud: true
    )
    
    pdf = CombinePDF.new
    pdf << CombinePDF.parse(grover_front.to_pdf)
    pdf << CombinePDF.parse(grover.to_pdf)
    pdf = pdf.to_pdf
    pdf
  end
end