#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class CheckReportsJob < Struct.new(:user, :verbose)

      def perform
        now    = Time.current
        base   = now.beginning_of_minute
        offset = (10 - base.min % 10) % 10
        current_slot = (base + offset.minutes).strftime('%H:%M')

        Report.find_each do |report|
          day = if report.schedule_type == "weekly"
            now.wday.zero? ? "7" : now.wday.to_s
          else
            now.day == now.end_of_month.day ? "28" : now.day.to_s
          end

          days = report.schedule_days.to_s.split(',')
          if report.schedule_hour == current_slot && days.include?(day)
            Redborder::Jobs::SendReportJob.new(report.id, user, 'dashboard', true).perform
          end
        end

        IncidentGroupReport.find_each do |report|

          day = case report.schedule_type
            when "daily"
              "any"
            when "weekly"
              now.wday.zero? ? "7" : now.wday.to_s
            when "monthly"
              now.day == now.end_of_month.day ? "28" : now.day.to_s
            else
              nil
          end

          days = report.schedule_days.to_s.split(',')
          
          if (report.schedule_hour == current_slot && (day == "any" || days.include?(day)))
            Redborder::Jobs::SendReportJob.new(report.id, user, 'incident', true).perform
          end
        end

        # Enqueue the job again
        t = Time.current.beginning_of_minute
        offset = 10 - t.min % 10
        run_at = t + offset.minutes
        Delayed::Job.enqueue(Redborder::Jobs::CheckReportsJob.new(user, true), priority: 10, run_at: run_at)
      end
    end
  end
end